@@ -57,7 +57,7 @@ static char *unsquashfs_options[]={
 	"-no-exit-code", "", "", "",
 	/* help options */
 	"-help", "-help-option", "-help-section", "-help-all", "-ho", "-hs",
-	"-ha", "", "", "",
+	"-ha", "-no-pager", "-cols", "", "", "",
 	/* misc options */
 	"-offset", "-fstime", "-ef", "-excf", "-L", "-pseudo-file", "", "", "",
 	NULL,
@@ -71,7 +71,7 @@ static char *sqfscat_options[]={
 	"-no-wildcards", "-regex", "", "", "",
 	/* help options */
 	"-help", "-help-option", "-help-section", "-help-all", "-ho", "-hs",
-	"-ha", NULL,
+	"-ha", "-no-pager", "-cols", NULL,
 };
 
 static char *unsquashfs_args[]={
@@ -86,7 +86,8 @@ static char *unsquashfs_args[]={
 	"", "<number>", "<size>", "<percent>", "", "", "", "", "", "", "", "",
 	"",
 	/* help options */
-	"", "<regex>", "<section>", "", "<regex>", "<section>", "", "", "", "",
+	"", "<regex>", "<section>", "", "<regex>", "<section>", "", "",
+	"<width>", "", "", "",
 	/* misc options */
 	"<bytes>", "", "<extract file>", "<exclude file>", "", "<file>", "",
 	"", "",
@@ -99,7 +100,8 @@ static char *sqfscat_args[]={
 	/* filter options */
 	"", "", "", "", "",
 	/* help options */
-	"", "<regex>", "<section>", "", "<regex>", "<section>", ""
+	"", "<regex>", "<section>", "", "<regex>", "<section>", "", "",
+	"<width>"
 };
 
 static char *unsquashfs_sections[]={
@@ -201,7 +203,8 @@ static char *unsquashfs_text[]={
 	"\t-no-exit[-code]\t\tdo not set exit code (to nonzero) on non-fatal "
 		"errors\n",
 	"\n", "Help options:", "\n",
-	"\t-h[elp]\t\t\tprint help summary information to stdout\n",
+	"\t-h[elp]\t\t\tprint help summary information to pager (or stdout if "
+		"not a terminal)\n",
 	"\t-help-option <regex>\tprint the help information for options "
 		"matching <regex> to pager (or stdout if not a terminal)\n",
 	"\t-help-section <section>\tprint the help information for section "
@@ -215,6 +218,9 @@ static char *unsquashfs_text[]={
 	"\t-ho <regex>\t\tshorthand alternative to -help-option\n",
 	"\t-hs <section>\t\tshorthand alternative to -help-section\n",
 	"\t-ha\t\t\tshorthand alternative to -help-all\n",
+	"\t-no-pager\t\tdo not use a pager to output help information\n",
+	"\t-cols <width>\t\tuse <width> columns to output help information.  "
+		"Useful if output is not to a terminal\n",
 	"\n", "Miscellaneous options:", "\n",
 	"\t-o[ffset] <bytes>\tskip <bytes> at start of FILESYSTEM.  Optionally "
 		"a suffix of K, M or G can be given to specify Kbytes, Mbytes "
@@ -244,12 +250,15 @@ static char *unsquashfs_text[]={
 	"\nSee -ignore-errors, -strict-errors and -no-exit-code options for "
 		"how they affect the exit status.\n",
 	"\n", "See also (extra information elsewhere):", "\n",
-	"The README for the Squashfs-tools 4.7 release, describing the new "
+	"The README for the Squashfs-tools 4.7.3 release, describing the new "
 		"features can be read here https://github.com/plougher/"
-		"squashfs-tools/blob/master/Documentation/4.7/README\n",
-	"\nThe Squashfs-tools USAGE guides and other documentation can be read "
-		"here https://github.com/plougher/squashfs-tools/blob/master/"
-		"Documentation/4.7\n",
+		"squashfs-tools/blob/master/Documentation/4.7.3/README.md\n",
+	"\nThe Unsquashfs usage guide can be read here https://github.com/"
+		"plougher/squashfs-tools/blob/master/Documentation/4.7.3/"
+		"USAGE-UNSQUASHFS.md\n",
+	"\nThe CHANGELOG for Squashfs tools can be read here "
+		"https://github.com/plougher/squashfs-tools/blob/master/"
+		"CHANGES.md\n",
 	"\n", "Decompressors available:", "\n",
 	"\t" DECOMPRESSORS "\n", NULL
 };
@@ -277,7 +286,8 @@ static char *sqfscat_text[]={
 	"\t-r[egex]\t\ttreat filenames as POSIX regular expressions rather "
 		"than use the default shell wildcard expansion (globbing)\n",
 	"\n", "Help options:", "\n",
-	"\t-h[elp]\t\t\tprint help summary information to stdout\n",
+	"\t-h[elp]\t\t\tprint help summary information to pager (or stdout if "
+		"not a terminal)\n",
 	"\t-help-option <regex>\tprint the help information for options "
 		"matching <regex> to pager (or stdout if not a terminal)\n",
 	"\t-help-section <section>\tprint the help information for section "
@@ -291,6 +301,9 @@ static char *sqfscat_text[]={
 	"\t-ho <regex>\t\tshorthand alternative to -help-option\n",
 	"\t-hs <section>\t\tshorthand alternative to -help-section\n",
 	"\t-ha\t\t\tshorthand alternative to -help-all\n",
+	"\t-no-pager\t\tdo not use a pager to output help information\n",
+	"\t-cols <width>\t\tuse <width> columns to output help information.  "
+		"Useful if output is not to a terminal\n",
 	"\n", "Environment:", "\n",
 	"\tSQFS_CMDLINE \t\tIf set, this is used as the directory to write the "
 		"file sqfs_cmdline which contains the command line arguments "
@@ -310,12 +323,15 @@ static char *sqfscat_text[]={
 	"\nSee -ignore-errors, -strict-errors and -no-exit-code options for "
 		"how they affect the exit status.\n",
 	"\n", "See also (extra information elsewhere):", "\n",
-	"The README for the Squashfs-tools 4.7 release, describing the new "
+	"The README for the Squashfs-tools 4.7.3 release, describing the new "
 		"features can be read here https://github.com/plougher/"
-		"squashfs-tools/blob/master/Documentation/4.7/README\n",
-	"\nThe Squashfs-tools USAGE guides and other documentation can be read "
-		"here https://github.com/plougher/squashfs-tools/blob/master/"
-		"Documentation/4.7\n",
+		"squashfs-tools/blob/master/Documentation/4.7.3/README.md\n",
+	"\nThe Sqfscat usage guide can be read here https://github.com/"
+		"plougher/squashfs-tools/blob/master/Documentation/4.7.3/"
+		"USAGE-SQFSCAT.md\n",
+	"\nThe CHANGELOG for Squashfs tools can be read here "
+		"https://github.com/plougher/squashfs-tools/blob/master/"
+		"CHANGES.md\n",
 	"\n", "Decompressors available:", "\n",
 	"\t" DECOMPRESSORS "\n", NULL,
 };
@@ -325,26 +341,14 @@ static void print_help_all(char *name, char *syntax, char **options_text)
 {
 	int i, cols;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	autowrap_printf(pager, cols, syntax, name);
 
 	for(i = 0; options_text[i] != NULL; i++)
 		autowrap_print(pager, options_text[i], cols);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(0);
 }
 
@@ -354,7 +358,7 @@ static void print_option(char *prog_name, char *opt_name, char *pattern, char **
 {
 	int i, res, matched = FALSE;
 	regex_t *preg = MALLOC(sizeof(regex_t));
-	int cols = get_column_width();
+	int cols;
 	pid_t pager_pid;
 	FILE *pager;
 
@@ -364,16 +368,12 @@ static void print_option(char *prog_name, char *opt_name, char *pattern, char **
 		char str[1024]; /* overflow safe */
 
 		regerror(res, preg, str, 1024);
+		cols = get_column_width();
 		autowrap_printf(stderr, cols, "%s: %s invalid regex %s because %s\n", prog_name, opt_name, pattern, str);
 		exit(1);
 	}
 
-	if(isatty(STDOUT_FILENO))
-		pager = exec_pager(&pager_pid);
-	else {
-		cols = 80;
-		pager = stdout;
-	}
+	pager = launch_pager(&pager_pid, &cols);
 
 	for(i = 0; options[i] != NULL; i++) {
 		res = regexec(preg, options[i], (size_t) 0, NULL, 0);
@@ -385,10 +385,7 @@ static void print_option(char *prog_name, char *opt_name, char *pattern, char **
 		}
 	}
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
+	delete_pager(pager, pager_pid);
 
 	if(!matched) {
 		autowrap_printf(stderr, cols, "%s: %s %s does not match any %s option\n", prog_name, opt_name, pattern, prog_name);
@@ -424,16 +421,8 @@ static void print_section(char *prog_name, char *opt_name, char *sec_name, char
 {
 	int i, j, secs, cols, res, matched = FALSE;
 	pid_t pager_pid;
-	FILE *pager;
 	regex_t *preg;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	if(strcmp(sec_name, "list") == 0) {
 		autowrap_printf(pager, cols, "\nUse following section name to print %s help information for that section\n\n", prog_name);
@@ -451,10 +440,7 @@ static void print_section(char *prog_name, char *opt_name, char *sec_name, char
 	if(res) {
 		char str[1024]; /* overflow safe */
 
-		if(pager != stdout) {
-			fclose(pager);
-			wait_to_die(pager_pid);
-		}
+		delete_pager(pager, pager_pid);
 
 		regerror(res, preg, str, 1024);
 		autowrap_printf(stderr, cols, "%s: %s invalid regex %s because %s\n", prog_name, opt_name, sec_name, str);
@@ -491,11 +477,7 @@ exact_match:
 	}
 
 finish:
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(0);
 }
 
@@ -504,15 +486,7 @@ static void handle_invalid_option(char *prog_name, char *opt_name, char **sectio
 {
 	int cols;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	autowrap_printf(pager, cols, "%s: %s is an invalid option\n\n", prog_name, opt_name);
 	autowrap_printf(pager, cols, "Run\n  \"%s -help-option <regex>\" to get help on all options matching <regex>\n", prog_name);
@@ -520,11 +494,7 @@ static void handle_invalid_option(char *prog_name, char *opt_name, char **sectio
 	print_section_names(pager, "\t", cols, sections, options_text);
 	autowrap_printf(pager, cols, "\nOr run\n  \"%s -help-all\" to get help on all the sections\n", prog_name);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(1);
 }
 
@@ -533,15 +503,7 @@ static void print_help(char *prog_name, char*message, char *syntax, char **secti
 {
 	int cols;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	if(message)
 		autowrap_print(pager, message, cols);
@@ -551,11 +513,7 @@ static void print_help(char *prog_name, char*message, char *syntax, char **secti
 	print_section_names(pager, "\t", cols, sections, options_text);
 	autowrap_printf(pager, cols, "\nOr run\n  \"%s -help-all\" to get help on all the sections\n", prog_name);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(message == NULL ? 0 : 1);
 }
 
@@ -565,15 +523,7 @@ static void print_option_help(char *prog_name, char *option, char **sections, ch
 	int cols;
 	char *string;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	VASPRINTF(&string, fmt, ap);
 	autowrap_print(pager, string, cols);
@@ -584,11 +534,7 @@ static void print_option_help(char *prog_name, char *option, char **sections, ch
 	autowrap_printf(pager, cols, "\nOr run\n  \"%s -help-all\" to get help on all the sections\n", prog_name);
 	free(string);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(1);
 }
 
